### 主题
Vue单元测试和组件通信


#### 单元测试

基于 vue-cli 3.0 的脚手架，然后 vue create name 创建一个文件名。推荐使用自己配置

测试框架选 mocha + chai 断言库 jest（我没有用过） 不需要手动引入断言库。


```
.
├── build                                       // webpack配置文件
├── src                                         // 源码目录
│  ├── components                               // 组件
│    
│── test
│   ├── specs								    // 测试脚本目录
│   |   ├──  HelloWorld.spec.js				    // 测试脚本 需要将测试代码放在这里
|   |   |
|   |   |
├── index.html                                  // 入口html文件
.

```


#### 测试脚本代码

假设有这样的Vue代码

```
<template>
  <div>{{msg}}</div>
</template>

<script>
export default {
  name: 'HelloWorld',
  
  props: {
    msg: String
  },
  
  data () {
    return {
      name: 'name'
    }
  }，
  
  methods: {
    setName(name) {
      this.name = name;
    }
  }
}
</script>

```





测试脚本里面包含一个或多个describe块 称为测试套件

```
import Vue from 'vue';
import HelloWorld from '@/component/HelloWorld.vue';

// data 属性
describe('HelloWorld.vue', () => {
  const Cotr = Vue.extend(HelloWorld);
  // 一个测试套件里面包含一个或多个it块 称为测试用例 
  it('renders props.msg when passed', () => {
    const vm = new Cotr().$mount();
    expect(vm.name).to.equal('name');
  });
    
  // methods 方法  
  it('给setMessage方法设置参数', () => {
    vm.setName('张三')
    expect(vm.name).to.equal('张三')
  });
  
  it('测试data数据', () => {
    const Cotr = Vue.extend(HelloWorld)
    const propData = { msg: 'hello' }
    const vm = new Cotr({ propsData: propData }).$mount()

    expect(vm.$el.textContent).to.equal('hello')
    
  })
  
  // 异步更新
  it('异步更新', done => {
    const vm = new Cotr().$mount();
    vm.msg = 'hello vue'
    Vue.nextTick(_ => {
      expect(vm.$el.textContent).to.equal('hello vue')
    })
    done()
  })
});

```



### 组件通信

主要介绍跨级的通信方法。 子传祖先和祖先传子孙。$on和$emit 联合起来使用。 
this.$emit 触发的自定义事件，然后在父组件上面监听这个自定义事件。这里看似是在父组件处理自定义事件
但是是在子组件上面触发的。所以我们完全可以在祖先组件上监听。


```
<template>
  <div class="child" @click="handleClick">
    我是子组件
  </div>
</template>

<script>
export default {
  name: 'child',
  
  methods: {
    handleClick() {
      this.$emit('on-message', 'ee');
    }
  },
  
  mount() {
    this.$on('on-message', (text) => {
      console.log(text);
    });
  },
};
</script>

```

类似上面的代码 我就可以在子孙组件找到祖先元素，然后触发自定义事件，在祖先组件里面监听自定义事件。


```
/**
 * @param { 组件名 } componentName 
 * @param { 事件名 } eventName 
 * @param { 参数 } params 
 */
function broadcast(componentName, eventName, params) {
  this.$children.forEach(child => {
    const name = child.$options.name;

    if (name === componentName) {
      child.$emit.apply(child, [eventName, params]);
    } else {
      broadcast.apply(child, [componentName, eventName].concat(params));
    }
  });
}

export default {
  methods: {
    // 派发 向上查找
    dispatch(componentName, eventName, params) {
      let parent = this.$parent || this.$root;
      let name = parent.$options.name;
      
      while (parent && (!name || name !== componentName)) {
        parent = parent.$parent;
        if (parent) {
          name = parent.$options.name;
        }
      }
      if (parent) {
        parent.$emit.apply(parent, [eventName].concat(params));
      }
    },
    // 广播 向下查找
    broadcast(componentName, eventName, params) {
      broadcast.call(this, componentName, eventName, params);
    }
  }
};


```

从上面的代码中 还可以自己扩展相关的方法。如果大家感兴趣，自己能够去实现一下。







